#!/usr/bin/python

#   * dome_track
#
#   * Check that the encoder monitor is running
#   * Check for lock file
#   * Create a lock file with a timestamp
#   * Open a connection to the dome switch
#   * Look up the right ascension and declination of the telescope field
#   * Calculate the optimal azimuth of the dome shutter
#   * Look up the current azimuth of the dome shutter
#   * Drive the dome to the optimal azimuth
#   * Remove the lock file

# Copyright (c) 2010-2015 John Kielkopf, Jeff Hay, Karen Collins, Rhodes Hart        
# kielkopf@louisville.edu                                                  
#                                                                          
#                                                                          
# Date:  December 6, 2015                                                  
# Version: 1.6
# For CDK700 at center of dome                                                            
#                                                                          
# This file is part of PyDome                                              


# Define values appropriate for this dome


# Switch settings for the DLI device

# CDK20N
# SWITCHIP="192.168.1.249"
# USER=""
# PASSWORD=""

# CDK700
SWITCHIP="192.168.10.252"
USER=""
PASSWORD=""


# CDK700
dps = 1.5



# Azimuth tolerance for setting

tolazimuth = 1.



################################################################################
################################################################################

import sys
import os
import time
from datetime import datetime
from math import sin, cos, asin, acos, atan2, sqrt, pi

# The dli module needs to be in the python path.  
# Add current directory to the path so we can find the dli module

sys.path.append('.')

import dli

if len(sys.argv) > 1:    
  sys.exit("Usage: dome_track")   

# Note starting time in Unix seconds and in UTC

timestamp = int(time.time())
startutc = time.strftime("%Y%m%d %H:%M:%S", time.gmtime())

# Test that the dome_monitor.py program is running

processname = 'dome_monitor.py'
processlist = os.popen("ps -Af").read()
proccount = processlist.count(processname)

if proccount > 1:
  print "%s is running %d processes\n" % (processname, proccount)
  exit()

if proccount == 0:
  print "%s is not running\n" % processname
  exit()


# Test for lockfile

lockfile = "/usr/local/observatory/status/domerotation"

if os.path.exists(lockfile):

  # Another process is running the dome
  lockfp = open(lockfile,'r')
  oldtime = int(float(lockfp.readline()))
  runtime = timestamp - oldtime
  print "Dome control in use: %d\n" % runtime
  exit()

else:

  # We take control and leave a message

  lockfp = open(lockfile,'w')
  lockfp.write('%d\n' % timestamp)
  lockfp.write('%s\n' % startutc)
  lockfp.close()
  
# Use the DLI switch to rotate the dome

# Open (connect) to the power switch, if the
# parameters are not passed it defaults to the
# IP address, user and password the switch is 
# set with from the factory.  Only changed items
# actually need to be passed.
switch=dli.powerswitch(hostname=SWITCHIP,userid=USER,password=PASSWORD)

# Verify the switch

if not switch.verify():
  print "Cannot talk to the switch"

  if os.path.exists(lockfile):

    # Remove the lock to allow future access
    os.remove(lockfile)
  
  exit()

# Map a time in hours to the range  0  to 24 

def map24(hour):
  
  if (hour < 0.0): 
    n = int(hour / 24.0) - 1
    hour24 = hour - n * 24.0
    return (hour24)
   
  elif (hour >= 24.0):   
    n = int(hour / 24.0)
    hour24= hour - n * 24.0
    return (hour24)
   
  else: 
    hour24 = hour
    return (hour24)


#Map an hourangle in hours to  -12 <= ha < +12 

def map12(hour):
  
  hour12=map24(hour)
  
  if (hour12 >= 12.0): 
    hour12 = hour12 - 24.
    return (hour12)
   
  else:   
    return (hour12)
  

# Map an angle in degrees to  0 <= angle < 360 

def map360(angle):

  if (angle < 0.0):   
    n = int(angle / 360.0) - 1
    return (angle - float(n) * 360.0)
   
  elif (angle >= 360.0):  
    n = int(angle / 360.0)
    return (angle - float(n) * 360.0)
   
  else:   
    return (angle)
  

# Map an angle in degrees to  -180 <= angle < 180 

def map180(angle):

  angle360 = map360(angle)
   
  if (angle360 >= 180.0):  
    return (angle360 - 360.0)
   
  else:   
    return (angle360)

    
# Find the fractional part handling negative values properly

def frac(x):

  newx = x - int(x)
  if (newx < 0.):
    newx = newx + 1.
      
  return (newx)


# Calculate the current universal time in hours with millisecond resolution

def utnow():
  
  dt = datetime.utcnow()
  
  hours = dt.hour
  minutes = dt.minute
  seconds = dt.second
  microseconds = dt.microsecond
  ut = hours + minutes/60. + seconds/3600. + microseconds/3600000000.
  
  return (ut)
    

# Calculate the Julian day number for a given year, month, day, and universal time

def calcjd(year,month,day,ut):
  
  day = day + ut / 24.0
  
  
  if int(month)== 1 or int(month)== 2:
    year = year - 1.
    month = month + 12.
  
  if (year + month/12.0 + day/365.25) >= (1582.0 + 10.0 / 12.0 + 15.0 / 365.25):
    na = int(year/100.)
    a = float(na)
    b = 2.0 - a + int(a/4.0)
  else:
    b = 0.
    
  if year < 0.:
    nc = int(float(int(365.25 * year)) - 0.75)
    c = float(nc)
  else:
    nc = int(365.25 * year)          
    c = float(nc)
  
  nd = int(30.6001 * (month + 1.)) 
  d = float(nd)
  jd = b + c + d + day + 1720994.5
  
  return (jd)


# Calculate the Julian day number at this moment

def jdnow():
  dt = datetime.utcnow()
  
  year = dt.year
  month = dt.month
  day = dt.day
  hours = dt.hour
  minutes = dt.minute
  seconds = dt.second
  microseconds = dt.microsecond
  ut = hours + minutes/60. + seconds/3600. + microseconds/3600000000.
  jd = calcjd(year, month, day, ut)
  
  return (jd)


# Calculate the sidereal time for a year, month, day and universal time at the define site latitude  
  
def calclst(year, month, day, ut, glong):
  jdeod =  calcjd(year, month, day, 0.0)
  tu = (jdeod - 2451545.0) / 36525.0
  t0 = (24110.54841 / 3600.0) + (8640184.812866 / 3600.0) * tu + (0.093104 / 3600.0) * tu * tu - (6.2e-6 / 3600.0) * tu * tu * tu
  t0 = map24(t0)
  gmst = map24(t0 + ut * 1.002737909)
  lmst = 24.0 * frac( (gmst - (glong / 15.0) ) / 24.0)
  
  return (lmst)
  

# Calculate the sidereal time at this moment for the defined site latitude

def lstnow():
  
  dt = datetime.utcnow()
  
  year = dt.year
  month = dt.month
  day = dt.day
  hours = dt.hour
  minutes = dt.minute
  seconds = dt.second
  microseconds = dt.microsecond
  ut = hours + minutes/60. + seconds/3600. + microseconds/3600000000.
  glong = site_longitude
  lst = calclst(year,month,day,ut,glong)
  
  return (lst)


# Convert local ha and dec to local az and alt  
# Geographic azimuth convention is followed: 
#   Due north is zero and azimuth increases from north to east 

def equatorial_to_horizontal(ha, dec):

  ha = ha*pi/12.
  phi = site_latitude*pi/180.
  dec = dec*pi/180.
  altitude = asin(sin(phi)*sin(dec)+cos(phi)*cos(dec)*cos(ha))
  altitude = altitude*180.0/pi
  azimuth = atan2(-cos(dec)*sin(ha),
    sin(dec)*cos(phi)-sin(phi)*cos(dec)*cos(ha))
  azimuth = azimuth*180.0/pi
  azimuth = map360(azimuth)

  return (azimuth, altitude)



# Look up the current telescope azimuth

# Open the telescope azimuth file
telfp = open("/usr/local/observatory/status/telaltaz", 'r')

# Read it  line by line into a list
teltext = telfp.readlines()

# Close the  file
telfp.close()

# Parse the data from the text

i = 0

for line in teltext:

  try:    
    # Treat the case of a plain text space separated entry
    entry = line.strip().split(" ")  
    # Get the values for this point 
    telaltitude = float(entry[0])
    telazimuth = float(entry[1])
    i = i + 1 

  except:
    if i == 0:
      
      print "No telescope data"
      exit()
    pass




newazimuth = map360(telazimuth)

# Print diagnostic
print ("New azimuth: %0.2f" % (newazimuth))

# Look up the current dome azimuth

nowazimuthstatus = open('/usr/local/observatory/status/domeazimuth','r')
nowazimuthstr = nowazimuthstatus.read(40)
nowazimuthstatus.close()
nowazimuth = float(nowazimuthstr)

# Print diagnostic
print ("Starting azimuth: %0.2f" % (nowazimuth))


# Set azimuth change 

delazimuth = map180(newazimuth - nowazimuth)


# This routine uses on/off and direction state relays

if abs(delazimuth) > tolazimuth:

  # Print the current state of all the outlets
  # switch.printstatus()
    
  if (delazimuth > 0.):
    
    # Power increase azimuth on
    switch.off(1)
    time.sleep(0.25)
    switch.on(2)
    starttime = time.time()

    # Print diagnostic
    # print 'Increasing dome azimuth'

    # Wait the dome to rotate the requested angle
    delay = delazimuth/dps
    time.sleep(delay)

    # Power increase dome azimuth off
    switch.off(1)
    switch.off(2)
    stoptime = time.time()

    # Print diagnostic
    # print 'Dome stopped'
    
    # Correct the azimuth based on the run time after the last tag    
    lastazimuthstatus = open('/usr/local/observatory/status/domeazimuth','r')
    lastazimuthstr = lastazimuthstatus.read(40)				        
    lastazimuthstatus.close()
    lastazimuth = float(lastazimuthstr)
    tagazimuthstatus = open('/usr/local/observatory/status/dometag','r')
    tagazimuthstr = tagazimuthstatus.read(40)				        
    tagazimuthstatus.close()
    tagtime = float(tagazimuthstr.split()[0])
    tagdelay = 0.
    if (starttime < tagtime):
      tagdelay = stoptime - tagtime         
    else:
      tagdelay = stoptime - starttime
    nowazimuth = map360(lastazimuth + tagdelay*dps)  
    
    # Update the azimuth at this instant    
    nowazimuthstatus = open('/usr/local/observatory/status/domeazimuth','w')    
    nowazimuthstatus.write("%f \n" % nowazimuth)
    nowazimuthstatus.close()


  elif (delazimuth < 0.):

    # Power decrease azimuth on
    switch.off(2)
    time.sleep(0.25)
    switch.on(1)
    starttime = time.time()

    # Print diagnostic
    # print 'Decreasing dome azimuth'

    # Wait for the dome to rotate the requested angle
    delay = -1.*delazimuth/dps
    time.sleep(delay)

    # Power decrease dome azimuth off
    switch.off(1)
    switch.off(2)
    stoptime = time.time()
    
    # Print diagnostic
    # print 'Dome stopped'
    
    # Correct the azimuth based on the run time after the last tag
    lastazimuthstatus = open('/usr/local/observatory/status/domeazimuth','r')
    lastazimuthstr = lastazimuthstatus.read(40)				        
    lastazimuthstatus.close()
    lastazimuth = float(lastazimuthstr)
    tagazimuthstatus = open('/usr/local/observatory/status/dometag','r')
    tagazimuthstr = tagazimuthstatus.read(60)				        
    tagazimuthstatus.close()
    tagtime = float(tagazimuthstr.split()[0])
    tagdelay = 0.
    if (starttime < tagtime):
      tagdelay = stoptime - tagtime         
    else:
      tagdelay = stoptime - starttime
    nowazimuth = map360(lastazimuth - tagdelay*dps)  
    
    # Update the azimuth at this instant    
    nowazimuthstatus = open('/usr/local/observatory/status/domeazimuth','w')    
    nowazimuthstatus.write("%f \n" % nowazimuth)
    nowazimuthstatus.close()
  
  else:
    pass

else:
  pass
  
print ("Current azimuth: %.2f" % (nowazimuth))

# Remove the lockfile

if os.path.exists(lockfile):

  # Remove the lock to allow future access
  os.remove(lockfile)

exit()
